module Midi.TestCoreMidi where
import Control.Monad
import qualified Control.Concurrent as Concurrent
import qualified Control.Concurrent.STM as STM
import qualified Data.Map as Map
import Data.Function
import Data.List

import qualified Midi.Midi as Midi
import qualified Midi.Parse as Parse
import qualified Midi.CoreMidi as CoreMidi

import Util.PPrint
import qualified Perform.Timestamp as Timestamp

-- TODO this needs a cleanup
-- make an interactive version to play around with
-- and non-interactive tests that can be run automatically
-- non-interactive should test the abort/pitchbend bug

main = CoreMidi.initialize test

test read_chan = do
    (rdevs, wdevs) <- CoreMidi.get_devices
    pprint rdevs
    pprint wdevs
    let out_dev = (Map.keys wdevs) !! 1

    putStrLn "connect"
    mapM_ (uncurry CoreMidi.connect_read_device) (Map.toList rdevs)

    let write_msg = make_write_msg wdevs

    now <- CoreMidi.now
    play_melody now write_msg out_dev
    -- sleep 0.9
    abort_at <- CoreMidi.now
    CoreMidi.abort -- (Map.elems wdevs !! 1)
    putStrLn $ "abort at " ++ show abort_at
    sleep 3
    -- monitor read_chan

    -- CoreMidi.write_message out_dev Timestamp.immediately
    --     (Midi.ChannelMessage 0 (Midi.NoteOn 74 66))
    -- write_sysex (iac 4) write_msg
    -- test_merge write_msg out_dev read_chan
    -- thru_loop write_msg out_dev read_chan

sleep = Concurrent.threadDelay . floor . (*1000000)

iac n = Midi.WriteDevice ("IAC Driver IAC Bus " ++ show n)

monitor read_chan = forever $ do
    Midi.ReadMessage (Midi.ReadDevice dev) (Timestamp.Timestamp ts) msg <-
        STM.atomically (STM.readTChan read_chan)
    print (ts, dev, msg)

test_merge write_msg out_dev read_chan = do
    Midi.ReadMessage _ ts _ <- STM.atomically (STM.readTChan read_chan)
    play_melody ts write_msg out_dev
    thru_melody write_msg out_dev

make_write_msg wdevs (dev, ts, msg) = do
    -- putStrLn $ "wr: " ++ show (dev, ts, msg)
    putStrLn $ show (ts, Parse.encode msg)
    CoreMidi.write_message (wdevs Map.! dev) ts msg

thru_loop write_msg wdev read_chan = forever $ do
    Midi.ReadMessage dev ts msg <- STM.atomically (STM.readTChan read_chan)
    print (ts, dev, msg)
    let msgs = thru (ts, msg)
    sequence_ [write_msg (wdev, ts, msg) | (ts, msg) <- msgs]

thru (ts, msg) = case msg of
    Midi.ChannelMessage ch (Midi.NoteOn key vel)
        -> [(ts1, Midi.ChannelMessage ch (Midi.NoteOn key vel))]

    Midi.ChannelMessage ch (Midi.NoteOff key vel)
        -> [(ts1, Midi.ChannelMessage ch (Midi.NoteOff key vel))]
    _ -> []
    where
    ts1 = ts -- + Timestamp.Timestamp 500

write_sysex wdev write_msg = do
    write_msg (wdev, Timestamp.immediately, big_sysex)

big_sysex = Midi.CommonMessage
    (Midi.SystemExclusive 42 (take 10000 (cycle [0..9]) ++ [0xf7]))


thru_melody write_msg out_dev = do
    let play msg = write_msg (mknote out_dev (0, msg))
    let notes = [70, 68 .. 60]
    forM_ (zip (notes++[0]) (head notes : notes)) $ \(key, last) -> do
        play (Midi.NoteOff last 0)
        when (key /= 0) $
            play (Midi.NoteOn key 100)
        Concurrent.threadDelay (4 * 100000)

play_melody ts_offset write_msg wdev =
    mapM_ write_msg (map (add_ts ts_offset) msgs)
    where
    notes = take 6 (melody wdev)
    pbs = [(1000, Midi.PitchBend 0.2)]
    msgs = sortBy (compare `on` \(_, t, _) -> t) $
        map (mknote wdev) (notes ++ pbs)

melody wdev = concat
    [[(ts, Midi.NoteOn key vel), (ts+400, Midi.NoteOff key vel)]
    | (ts, key) <- zip [0, 500..] score]
    where
    vel = 70
    score = [53, 55 .. 61]

mknote wdev (ts, msg) =
    (wdev, Timestamp.Timestamp ts, Midi.ChannelMessage 0 msg)

add_ts ts (dev, msg_ts, msg) = (dev, ts + msg_ts, msg)
